<?xml version="1.0" encoding="UTF-8"?>
<gui name="konquest"
     version="4"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">

<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
  <Action name="game_measure"/>
  <Action name="game_fleets"/>
  </Menu>
  <Menu name="view"><text>&amp;View</text>
  <Action name="view_messages"/>
  <Action name="view_standings"/>
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="game_new"/>
  <Action name="game_end"/>
  <Separator/>
  <Action name="move_end_turn"/>
  <Separator/>
  <Action name="game_measure"/>
  <Action name="view_standings"/>
  <Action name="game_fleets"/>
</ToolBar>

</gui>
